@@ -34,7 +34,7 @@ function sin(x::T) where T<:Union{Float32, Float64}
         end
         return sin_kernel(x)
     elseif isnan(x)
-        return T(NaN)
+        return x
     elseif isinf(x)
         sin_domain_error(x)
     end
@@ -103,7 +103,7 @@ function cos(x::T) where T<:Union{Float32, Float64}
         end
         return cos_kernel(x)
     elseif isnan(x)
-        return T(NaN)
+        return x
     elseif isinf(x)
         cos_domain_error(x)
     else
@@ -127,6 +127,7 @@ const DC3 = 2.48015872894767294178e-05
 const DC4 = -2.75573143513906633035e-07
 const DC5 = 2.08757232129817482790e-09
 const DC6 = -1.13596475577881948265e-11
+
 """
     cos_kernel(y)
 
@@ -168,6 +169,8 @@ end
 
 Simultaneously compute the sine and cosine of `x`, where `x` is in radians, returning
 a tuple `(sine, cosine)`.
+
+See also [`cis`](@ref), [`sincospi`](@ref), [`sincosd`](@ref).
 """
 function sincos(x::T) where T<:Union{Float32, Float64}
     if abs(x) < T(pi)/4
@@ -176,7 +179,7 @@ function sincos(x::T) where T<:Union{Float32, Float64}
         end
         return sincos_kernel(x)
     elseif isnan(x)
-        return T(NaN), T(NaN)
+        return x, x
     elseif isinf(x)
         sincos_domain_error(x)
     end
@@ -218,7 +221,7 @@ function tan(x::T) where T<:Union{Float32, Float64}
         end
         return tan_kernel(x)
     elseif isnan(x)
-        return T(NaN)
+        return x
     elseif isinf(x)
         tan_domain_error(x)
     end
@@ -253,7 +256,7 @@ end
     #
     #       Note: tan(y+z) = tan(y) + tan'(y)*z
     #                  ~ tan(y) + (1+y*y)*z
-    #       Therefore, for better accuracz in computing tan(y+z), let
+    #       Therefore, for better accuracy in computing tan(y+z), let
     #             3      2      2       2       2
     #        r = y *(T2+y *(T3+y *(...+y *(T12+y *T13))))
     #       then
@@ -421,7 +424,7 @@ end
     flipsign(Float32(pi/2 - 2*(s + s*tRt)), x)
 end
 
-@noinline asin_domain_error(x) = throw(DomainError(x, "asin(x) is not defined for |x|>1."))
+@noinline asin_domain_error(x) = throw(DomainError(x, "asin(x) is not defined for |x| > 1."))
 function asin(x::T) where T<:Union{Float32, Float64}
     # Since  asin(x) = x + x^3/6 + x^5*3/40 + x^7*15/336 + ...
     # we approximate asin(x) on [0,0.5] by
@@ -579,8 +582,8 @@ function atan(y::T, x::T) where T<:Union{Float32, Float64}
     #    S8) ATAN2(+-INF,+INF ) is +-pi/4 ;
     #    S9) ATAN2(+-INF,-INF ) is +-3pi/4;
     #    S10) ATAN2(+-INF, (anything but,0,NaN, and INF)) is +-pi/2;
-    if isnan(x) || isnan(y) # S1 or S2
-        return T(NaN)
+    if isnan(x) | isnan(y) # S1 or S2
+        return isnan(x) ? x : y
     end
 
     if x == T(1.0) # then y/x = y and x > 0, see M2
@@ -720,141 +723,124 @@ function acos(x::T) where T <: Union{Float32, Float64}
     end
 end
 
-# multiply in extended precision
-function mulpi_ext(x::Float64)
-    m = 3.141592653589793
-    m_hi = 3.1415926218032837
-    m_lo = 3.178650954705639e-8
+# Uses minimax polynomial of sin(π * x) for π * x in [0, .25]
+@inline function sinpi_kernel(x::Float64)
+    sinpi_kernel_wide(x)
+end
+@inline function sinpi_kernel_wide(x::Float64)
+    x² = x*x
+    x⁴ = x²*x²
+    r  = evalpoly(x², (2.5501640398773415, -0.5992645293202981, 0.08214588658006512,
+                       -7.370429884921779e-3, 4.662827319453555e-4, -2.1717412523382308e-5))
+    return muladd(3.141592653589793, x, x*muladd(-5.16771278004997,
+                  x², muladd(x⁴, r,  1.2245907532225998e-16)))
+end
+@inline function sinpi_kernel(x::Float32)
+    Float32(sinpi_kernel_wide(x))
+end
+@inline function sinpi_kernel_wide(x::Float32)
+    x = Float64(x)
+    return x*evalpoly(x*x, (3.1415926535762266, -5.167712769188119,
+                            2.5501626483206374, -0.5992021090314925, 0.08100185277841528))
+end
 
-    x_hi = reinterpret(Float64, reinterpret(UInt64,x) & 0xffff_ffff_f800_0000)
-    x_lo = x-x_hi
+@inline function sinpi_kernel(x::Float16)
+    Float16(sinpi_kernel_wide(x))
+end
+@inline function sinpi_kernel_wide(x::Float16)
+    x = Float32(x)
+    return x*evalpoly(x*x, (3.1415927f0, -5.1677127f0, 2.5501626f0, -0.5992021f0, 0.081001855f0))
+end
 
-    y_hi = m*x
-    y_lo = x_hi * m_lo + (x_lo* m_hi + ((x_hi*m_hi-y_hi) + x_lo*m_lo))
+# Uses minimax polynomial of cos(π * x) for π * x in [0, .25]
+@inline function cospi_kernel(x::Float64)
+    cospi_kernel_wide(x)
+end
+@inline function cospi_kernel_wide(x::Float64)
+    x² = x*x
+    r = x²*evalpoly(x², (4.058712126416765, -1.3352627688537357, 0.23533063027900392,
+                         -0.025806887811869204, 1.9294917136379183e-3, -1.0368935675474665e-4))
+    a_x² = 4.934802200544679 * x²
+    a_x²lo = muladd(3.109686485461973e-16, x², muladd(4.934802200544679, x², -a_x²))
 
-    DoubleFloat64(y_hi,y_lo)
+    w  = 1.0-a_x²
+    return w + muladd(x², r, ((1.0-w)-a_x²) - a_x²lo)
+end
+@inline function cospi_kernel(x::Float32)
+    Float32(cospi_kernel_wide(x))
+end
+@inline function cospi_kernel_wide(x::Float32)
+    x = Float64(x)
+    return evalpoly(x*x, (1.0, -4.934802200541122, 4.058712123568637,
+                          -1.3352624040152927, 0.23531426791507182, -0.02550710082498761))
+end
+@inline function cospi_kernel(x::Float16)
+    Float16(cospi_kernel_wide(x))
+end
+@inline function cospi_kernel_wide(x::Float16)
+    x = Float32(x)
+    return evalpoly(x*x, (1.0f0, -4.934802f0, 4.058712f0, -1.3352624f0, 0.23531426f0, -0.0255071f0))
 end
-mulpi_ext(x::Float32) = DoubleFloat32(pi*Float64(x))
-mulpi_ext(x::Rational) = mulpi_ext(float(x))
-mulpi_ext(x::Real) = pi*x # Fallback
 
 """
     sinpi(x)
 
 Compute ``\\sin(\\pi x)`` more accurately than `sin(pi*x)`, especially for large `x`.
+
+See also [`sind`](@ref), [`cospi`](@ref), [`sincospi`](@ref).
 """
-function sinpi(x::T) where T<:AbstractFloat
+function sinpi(_x::T) where T<:IEEEFloat
+    x = abs(_x)
     if !isfinite(x)
         isnan(x) && return x
         throw(DomainError(x, "`x` cannot be infinite."))
     end
+    # For large x, answers are all 1 or zero.
+    x >= maxintfloat(T) && return copysign(zero(T), _x)
 
-    ax = abs(x)
-    s = maxintfloat(T)/2
-    ax >= s && return copysign(zero(T),x) # integer-valued
-
-    # reduce to interval [-1,1]
-    # assumes RoundNearest rounding mode
-    t = 3*s
-    rx = x-((x+t)-t) # zeros may be incorrectly signed
-    arx = abs(rx)
-
-    if (arx == 0) | (arx == 1)
-        copysign(zero(T),x)
-    elseif arx < 0.25
-        sin_kernel(mulpi_ext(rx))
-    elseif arx < 0.75
-        y = mulpi_ext(T(0.5) - arx)
-        copysign(cos_kernel(y),rx)
+    # reduce to interval [0, 0.5]
+    n = round(2*x)
+    rx = float(muladd(T(-.5), n, x))
+    n = Int64(n) & 3
+    if n==0
+        res = sinpi_kernel(rx)
+    elseif n==1
+        res = cospi_kernel(rx)
+    elseif n==2
+        res = zero(T)-sinpi_kernel(rx)
     else
-        y = mulpi_ext(copysign(one(T),rx) - rx)
-        sin_kernel(y)
+        res = zero(T)-cospi_kernel(rx)
     end
+    return ifelse(signbit(_x), -res, res)
 end
-
-# Rationals
-function sinpi(x::T) where T<:Rational
-    Tf = float(T)
-    if !isfinite(x)
-        throw(DomainError(x, "`x` must be finite."))
-    end
-
-    # until we get an IEEE remainder function (#9283)
-    rx = rem(x,2)
-    if rx > 1
-        rx -= 2
-    elseif rx < -1
-        rx += 2
-    end
-    arx = abs(rx)
-
-    if (arx == 0) | (arx == 1)
-        copysign(zero(Tf),x)
-    elseif arx < 0.25
-        sin_kernel(mulpi_ext(rx))
-    elseif arx < 0.75
-        y = mulpi_ext(T(0.5) - arx)
-        copysign(cos_kernel(y),rx)
-    else
-        y = mulpi_ext(copysign(one(T),rx) - rx)
-        sin_kernel(y)
-    end
-end
-
 """
     cospi(x)
 
 Compute ``\\cos(\\pi x)`` more accurately than `cos(pi*x)`, especially for large `x`.
 """
-function cospi(x::T) where T<:AbstractFloat
+function cospi(x::T) where T<:IEEEFloat
+    x = abs(x)
     if !isfinite(x)
         isnan(x) && return x
         throw(DomainError(x, "`x` cannot be infinite."))
     end
+    # For large x, answers are all 1 or zero.
+    x >= maxintfloat(T) && return one(T)
 
-    ax = abs(x)
-    s = maxintfloat(T)
-    ax >= s && return one(T) # even integer-valued
-
-    # reduce to interval [-1,1], then [0,1]
-    # assumes RoundNearest rounding mode
-    rx = abs(ax-((ax+s)-s))
-
-    if rx <= 0.25
-        cos_kernel(mulpi_ext(rx))
-    elseif rx < 0.75
-        y = mulpi_ext(T(0.5) - rx)
-        sin_kernel(y)
+    # reduce to interval [0, 0.5]
+    n = round(2*x)
+    rx = float(muladd(T(-.5), n, x))
+    n = Int64(n) & 3
+    if n==0
+        return cospi_kernel(rx)
+    elseif n==1
+        return zero(T)-sinpi_kernel(rx)
+    elseif n==2
+        return zero(T)-cospi_kernel(rx)
     else
-        y = mulpi_ext(one(T) - rx)
-        -cos_kernel(y)
+        return sinpi_kernel(rx)
     end
 end
-
-# Rationals
-function cospi(x::T) where T<:Rational
-    if !isfinite(x)
-        throw(DomainError(x, "`x` must be finite."))
-    end
-
-    ax = abs(x)
-    # until we get an IEEE remainder function (#9283)
-    rx = rem(ax,2)
-    if rx > 1
-        rx = 2-rx
-    end
-
-    if rx <= 0.25
-        cos_kernel(mulpi_ext(rx))
-    elseif rx < 0.75
-        y = mulpi_ext(T(0.5) - rx)
-        sin_kernel(y)
-    else
-        y = mulpi_ext(one(T) - rx)
-        -cos_kernel(y)
-    end
-end
-
 """
     sincospi(x)
 
@@ -863,75 +849,86 @@ where `x` is in radians), returning a tuple `(sine, cosine)`.
 
 !!! compat "Julia 1.6"
     This function requires Julia 1.6 or later.
+
+See also: [`cispi`](@ref), [`sincosd`](@ref), [`sinpi`](@ref).
 """
-function sincospi(x::T) where T<:AbstractFloat
+function sincospi(_x::T) where T<:IEEEFloat
+    x = abs(_x)
     if !isfinite(x)
         isnan(x) && return x, x
         throw(DomainError(x, "`x` cannot be infinite."))
     end
+    # For large x, answers are all 1 or zero.
+    x >= maxintfloat(T) && return (copysign(zero(T), _x), one(T))
 
-    ax = abs(x)
-    s = maxintfloat(T)
-    ax >= s && return (copysign(zero(T), x), one(T)) # even integer-valued
-
-    # reduce to interval [-1,1]
-    # assumes RoundNearest rounding mode
-    t = 3*(s/2)
-    rx = x-((x+t)-t) # zeros may be incorrectly signed
-    arx = abs(rx)
-
-    # same selection scheme as sinpi and cospi
-    if (arx == 0) | (arx == 1)
-        return copysign(zero(T), x), ifelse(ax % 2 == 0, one(T), -one(T))
-    elseif arx < 0.25
-        return sincos_kernel(mulpi_ext(rx))
-    elseif arx < 0.75
-        y = mulpi_ext(T(0.5) - arx)
-        return copysign(cos_kernel(y), rx), sin_kernel(y)
+    # reduce to interval [0, 0.5]
+    n = round(2*x)
+    rx = float(muladd(T(-.5), n, x))
+    n = Int64(n) & 3
+    si, co = sinpi_kernel(rx),cospi_kernel(rx)
+    if n==0
+        si, co = si, co
+    elseif n==1
+        si, co  = co, zero(T)-si
+    elseif n==2
+        si, co  = zero(T)-si, zero(T)-co
     else
-        y_si = mulpi_ext(copysign(one(T), rx) - rx)
-        y_co = mulpi_ext(one(T) - arx)
-        return sin_kernel(y_si), -cos_kernel(y_co)
+        si, co  = zero(T)-co, si
     end
+    si = ifelse(signbit(_x), -si, si)
+    return si, co
 end
 
-# Rationals
-function sincospi(x::T) where T<:Rational
-    Tf = float(T)
+"""
+    tanpi(x)
+
+Compute ``\\tan(\\pi x)`` more accurately than `tan(pi*x)`, especially for large `x`.
+
+!!! compat "Julia 1.10"
+    This function requires at least Julia 1.10.
+
+See also [`tand`](@ref), [`sinpi`](@ref), [`cospi`](@ref), [`sincospi`](@ref).
+"""
+function tanpi(_x::T) where T<:IEEEFloat
+    # This is modified from sincospi.
+    # Would it be faster or more accurate to make a tanpi_kernel?
+    x = abs(_x)
     if !isfinite(x)
-        throw(DomainError(x, "`x` must be finite."))
+        isnan(x) && return x
+        throw(DomainError(x, "`x` cannot be infinite."))
     end
+    # For large x, answers are all zero.
+    # All integer values for floats larger than maxintfloat are even.
+    x >= maxintfloat(T) && return copysign(zero(T), _x)
 
-    # until we get an IEEE remainder function (#9283)
-    rx = rem(x,2)
-    if rx > 1
-        rx -= 2
-    elseif rx < -1
-        rx += 2
-    end
-    arx = abs(rx)
-
-    # same selection scheme as sinpi and cospi
-    if (arx == 0) | (arx == 1)
-        return copysign(zero(Tf),x), ifelse(iseven(numerator(x)), one(Tf), -one(Tf))
-    elseif arx < 0.25
-        return sincos_kernel(mulpi_ext(rx))
-    elseif arx < 0.75
-        y = mulpi_ext(T(0.5) - arx)
-        return copysign(cos_kernel(y), rx), sin_kernel(y)
+    # reduce to interval [0, 0.5]
+    n = round(2*x)
+    rx = float(muladd(T(-.5), n, x))
+    n = Int64(n) & 3
+    si, co = sinpi_kernel_wide(rx), cospi_kernel_wide(rx)
+    if n==0
+        si, co = si, co
+    elseif n==1
+        si, co  = co, zero(T)-si
+    elseif n==2
+        si, co  = zero(T)-si, zero(T)-co
     else
-        y_si = mulpi_ext(copysign(one(T), rx) - rx)
-        y_co = mulpi_ext(one(T) - arx)
-        return sin_kernel(y_si), -cos_kernel(y_co)
+        si, co  = zero(T)-co, si
     end
+    si = ifelse(signbit(_x), -si, si)
+    return float(T)(si / co)
 end
 
 sinpi(x::Integer) = x >= 0 ? zero(float(x)) : -zero(float(x))
 cospi(x::Integer) = isodd(x) ? -one(float(x)) : one(float(x))
+tanpi(x::Integer) = x >= 0 ? (isodd(x) ? -zero(float(x)) : zero(float(x))) :
+                             (isodd(x) ? zero(float(x)) : -zero(float(x)))
 sincospi(x::Integer) = (sinpi(x), cospi(x))
-sinpi(x::Real) = sinpi(float(x))
-cospi(x::Real) = cospi(float(x))
-sincospi(x::Real) = sincospi(float(x))
+sinpi(x::AbstractFloat) = sin(pi*x)
+cospi(x::AbstractFloat) = cos(pi*x)
+sincospi(x::AbstractFloat) = sincos(pi*x)
+tanpi(x::AbstractFloat) = tan(pi*x)
+tanpi(x::Complex) = sinpi(x) / cospi(x) # Is there a better way to do this?
 
 function sinpi(z::Complex{T}) where T
     F = float(T)
@@ -1068,7 +1065,9 @@ isinf_real(x::Number) = false
 """
     sinc(x)
 
-Compute ``\\sin(\\pi x) / (\\pi x)`` if ``x \\neq 0``, and ``1`` if ``x = 0``.
+Compute normalized sinc function ``\\operatorname{sinc}(x) = \\sin(\\pi x) / (\\pi x)`` if ``x \\neq 0``, and ``1`` if ``x = 0``.
+
+See also [`cosc`](@ref), its derivative.
 """
 sinc(x::Number) = _sinc(float(x))
 sinc(x::Integer) = iszero(x) ? one(x) : zero(x)
@@ -1085,6 +1084,8 @@ _sinc(x::ComplexF16) = ComplexF16(_sinc(ComplexF32(x)))
 
 Compute ``\\cos(\\pi x) / x - \\sin(\\pi x) / (\\pi x^2)`` if ``x \\neq 0``, and ``0`` if
 ``x = 0``. This is the derivative of `sinc(x)`.
+
+See also [`sinc`](@ref).
 """
 cosc(x::Number) = _cosc(float(x))
 function _cosc(x::Number)
@@ -1183,7 +1184,7 @@ function sind(x::Real)
     if isinf(x)
         return throw(DomainError(x, "`x` cannot be infinite."))
     elseif isnan(x)
-        return oftype(x,NaN)
+        return x
     end
 
     rx = copysign(float(rem(x,360)),x)
@@ -1214,7 +1215,7 @@ function cosd(x::Real)
     if isinf(x)
         return throw(DomainError(x, "`x` cannot be infinite."))
     elseif isnan(x)
-        return oftype(x,NaN)
+        return x
     end
 
     rx = abs(float(rem(x,360)))
@@ -1246,37 +1247,45 @@ Simultaneously compute the sine and cosine of `x`, where `x` is in degrees.
 !!! compat "Julia 1.3"
     This function requires at least Julia 1.3.
 """
-function sincosd(x::Real)
-    if isinf(x)
-        return throw(DomainError(x, "sincosd(x) is only defined for finite `x`."))
-    elseif isnan(x)
-        return (oftype(x,NaN), oftype(x,NaN))
-    end
-
-    # It turns out that calling those functions separately yielded better
-    # performance than considering each case and calling `sincos_kernel`.
-    return (sind(x), cosd(x))
-end
+sincosd(x) = (sind(x), cosd(x))
+# It turns out that calling these functions separately yields better
+# performance than considering each case and calling `sincos_kernel`.
 
 sincosd(::Missing) = (missing, missing)
 
 for (fd, f, fn) in ((:sind, :sin, "sine"), (:cosd, :cos, "cosine"), (:tand, :tan, "tangent"))
-    name = string(fd)
-    @eval begin
-        @doc """
-            $($name)(x)
-        Compute $($fn) of `x`, where `x` is in degrees. """ ($fd)(z) = ($f)(deg2rad(z))
+    for (fu, un) in ((:deg2rad, "degrees"),)
+        name = string(fd)
+        @eval begin
+            @doc """
+                $($name)(x)
+
+            Compute $($fn) of `x`, where `x` is in $($un).
+            If `x` is a matrix, `x` needs to be a square matrix.
+
+            !!! compat "Julia 1.7"
+                Matrix arguments require Julia 1.7 or later.
+            """ ($fd)(x) = ($f)(($fu).(x))
+        end
     end
 end
 
 for (fd, f, fn) in ((:asind, :asin, "sine"), (:acosd, :acos, "cosine"),
                     (:asecd, :asec, "secant"), (:acscd, :acsc, "cosecant"), (:acotd, :acot, "cotangent"))
-    name = string(fd)
-    @eval begin
-        @doc """
-            $($name)(x)
 
-        Compute the inverse $($fn) of `x`, where the output is in degrees. """ ($fd)(y) = rad2deg(($f)(y))
+    for (fu, un) in ((:rad2deg, "degrees"),)
+        name = string(fd)
+        @eval begin
+            @doc """
+                $($name)(x)
+
+            Compute the inverse $($fn) of `x`, where the output is in $($un).
+            If `x` is a matrix, `x` needs to be a square matrix.
+
+            !!! compat "Julia 1.7"
+                Matrix arguments require Julia 1.7 or later.
+            """ ($fd)(x) = ($fu).(($f)(x))
+        end
     end
 end
 
@@ -1285,6 +1294,9 @@ end
     atand(y,x)
 
 Compute the inverse tangent of `y` or `y/x`, respectively, where the output is in degrees.
+
+!!! compat "Julia 1.7"
+    The one-argument method supports square matrix arguments as of Julia 1.7.
 """
-atand(y)    = rad2deg(atan(y))
-atand(y, x) = rad2deg(atan(y,x))
+atand(y)    = rad2deg.(atan(y))
+atand(y, x) = rad2deg.(atan(y,x))
