@@ -201,11 +201,11 @@ include("setup.jl")
         end
 
         @testset "delete default header" begin
-        headers = [
-            "Accept"     => nothing
-            "User-Agent" => nothing
-        ]
-        json = download_json(url, headers = headers)
+            headers = [
+                "Accept"     => nothing
+                "User-Agent" => nothing
+            ]
+            json = download_json(url, headers = headers)
             @test !("Accept" in keys(json["headers"]))
             @test !("User-Agent" in keys(json["headers"]))
         end
@@ -228,7 +228,7 @@ include("setup.jl")
         resp = request(url, debug = (type, msg) -> push!(events, type => msg))
         @test resp isa Response && resp.status == 200
         @test any(events) do (type, msg)
-            type == "TEXT" && startswith(msg, "Connected to ")
+            type == "TEXT" && startswith(msg, r"(Connected to |Connection.* left intact)")
         end
         @test any(events) do (type, msg)
             type == "HEADER OUT" && contains(msg, r"^HEAD /get HTTP/[\d\.+]+\s$"m)
@@ -253,6 +253,32 @@ include("setup.jl")
         @test isempty(cookies)
     end
 
+    @testset "default_downloader!" begin
+        original = Downloads.DOWNLOADER[]
+        try
+            tripped = false
+            url = "$server/get"
+
+            downloader = Downloader()
+            downloader.easy_hook = (easy, info) -> tripped = true
+
+            # set default
+            default_downloader!(downloader)
+            _ = download_body(url)
+            @test tripped
+
+            #reset tripwire
+            tripped = false
+
+            # reset default
+            default_downloader!()
+            _ = download_body(url)
+            @test !tripped
+        finally
+            Downloads.DOWNLOADER[] = original
+        end
+    end
+
     @testset "netrc support" begin
         user = "gVvkQiHN62"
         passwd = "dlctfSMTno8n"
@@ -353,25 +379,31 @@ include("setup.jl")
     end
 
     @testset "concurrent requests" begin
-        mine = Downloader()
-        for downloader in (nothing, mine)
-            have_lsof = Sys.which("lsof") !== nothing
-            count_tcp() = Base.count(x->contains("TCP",x), split(read(`lsof -p $(getpid())`, String), '\n'))
-            if have_lsof
-                n_tcp = count_tcp()
-            end
-            delay = 2
-            count = 100
-            url = "$server/delay/$delay"
-            t = @elapsed @sync for id = 1:count
-                @async begin
-                    json = download_json("$url?id=$id", downloader = downloader)
-                    @test get(json["args"], "id", nothing) == ["$id"]
+        if Sys.iswindows()
+            # Known issue https://github.com/JuliaLang/Downloads.jl/issues/227
+            # These test should be fixed on Windows and then reenabled.
+            @test_skip "concurrent requests flakey on Windows"
+        else
+            mine = Downloader()
+            for downloader in (nothing, mine)
+                have_lsof = Sys.which("lsof") !== nothing
+                count_tcp() = Base.count(x->contains("TCP",x), split(read(`lsof -p $(getpid())`, String), '\n'))
+                if have_lsof
+                    n_tcp = count_tcp()
+                end
+                delay = 2
+                count = 100
+                url = "$server/delay/$delay"
+                t = @elapsed @sync for id = 1:count
+                    @async begin
+                        json = download_json("$url?id=$id", downloader = downloader)
+                        @test get(json["args"], "id", nothing) == ["$id"]
+                    end
+                end
+                @test t < 0.9*count*delay
+                if have_lsof
+                    @test n_tcp == count_tcp()
                 end
-            end
-            @test t < 0.9*count*delay
-            if have_lsof
-                @test n_tcp == count_tcp()
             end
         end
     end
@@ -436,8 +468,31 @@ include("setup.jl")
             end
         end
 
+        @testset "interrupt" begin
+            url = "$server/delay/10"
+            interrupt = Base.Event()
+            download_task = @async request(url; interrupt=interrupt)
+            sleep(0.1)
+            @test !istaskdone(download_task)
+            notify(interrupt)
+            timedwait(()->istaskdone(download_task), 5.0)
+            @test istaskdone(download_task)
+            @test download_task.result isa RequestError
+
+            interrupt = Base.Event()
+            url = "$server/put"
+            input=`sh -c 'sleep 15; echo "hello"'`
+            download_task = @async request(url; interrupt=interrupt, input=input)
+            sleep(0.1)
+            @test !istaskdone(download_task)
+            notify(interrupt)
+            timedwait(()->istaskdone(download_task), 5.0)
+            @test istaskdone(download_task)
+            @test download_task.result isa RequestError
+        end
+
         @testset "progress" begin
-            url = "https://httpbingo.org/drip"
+            url = "$server/drip"
             progress = []
             dl_funcs = [
                 download,
@@ -547,8 +602,8 @@ include("setup.jl")
 
     @testset "grace cleanup" begin
         dl = Downloader(grace=1)
-        Downloads.download("https://httpbingo.org/drip"; downloader=dl)
-        Downloads.download("https://httpbingo.org/drip"; downloader=dl)
+        Downloads.download("$server/drip"; downloader=dl)
+        Downloads.download("$server/drip"; downloader=dl)
     end
 
     @testset "Input body size" begin
