@@ -1,17 +1,20 @@
 # This file is a part of Julia. License is MIT: https://julialang.org/license
 
 module ResolveTest
+import ..Pkg # ensure we are using the correct Pkg
 
 using Test
 using Pkg.Types
-using Pkg.GraphType
 using Pkg.Types: VersionBound
+using UUIDs
 using Pkg.Resolve
-using Pkg.Resolve.VersionWeights
-import Pkg.Types: uuid5, uuid_package
+import Pkg.Resolve: VersionWeight, add_reqs!, simplify_graph!, ResolverError, Fixed, Requires
+import ..HistoricalStdlibVersions
 
-# print info, stats etc.
-const VERBOSE = false
+include("utils.jl")
+using .Utils
+include("resolve_utils.jl")
+using .ResolveUtils
 
 # Check that VersionWeight keeps the same ordering as VersionNumber
 
@@ -35,169 +38,6 @@ for v1 in vlst, v2 in vlst
     @test ceq == (vw1 == vw2)
 end
 
-# auxiliary functions
-pkguuid(p::String) = uuid5(uuid_package, p)
-function storeuuid(p::String, uuid_to_name::Dict{UUID,String})
-    uuid = p == "julia" ? Types.uuid_julia : pkguuid(p)
-    if haskey(uuid_to_name, uuid)
-        @assert uuid_to_name[uuid] == p
-    else
-        uuid_to_name[uuid] = p
-    end
-    return uuid
-end
-wantuuids(want_data) = Dict{UUID,VersionNumber}(pkguuid(p) => v for (p,v) in want_data)
-
-function load_package_data_raw(T::Type, input::String)
-    toml = Types.TOML.parse(input)
-    data = Dict{VersionRange,Dict{String,T}}()
-    for (v, d) in toml, (key, value) in d
-        vr = VersionRange(v)
-        dict = get!(data, vr, Dict{String,T}())
-        haskey(dict, key) && pkgerror("$ver/$key is duplicated in $path")
-        dict[key] = T(value)
-    end
-    return data
-end
-
-function gen_versionranges(dict::Dict{K,Set{VersionNumber}}, srtvers::Vector{VersionNumber}) where {K}
-    vranges = Dict{K,Vector{VersionRange}}()
-    for (vreq,vset) in dict
-        vranges[vreq] = VersionRange[]
-        while !isempty(vset)
-            vn0 = minimum(vset)
-            i = findfirst(isequal(vn0), srtvers)
-            @assert i ≠ 0
-            pop!(vset, vn0)
-            vn1 = vn0
-            pushed = false
-            j = i + 1
-            while j ≤ length(srtvers)
-                vn = srtvers[j]
-                if vn ∈ vset
-                    pop!(vset, vn)
-                    vn1 = vn
-                    j += 1
-                else
-                    # vn1 =  srtvers[j-1]
-                    push!(vranges[vreq], VersionRange(VersionBound(vn0),VersionBound(vn1)))
-                    pushed = true
-                    break
-                end
-            end
-            !pushed && push!(vranges[vreq], VersionRange(VersionBound(vn0),VersionBound(vn1)))
-        end
-    end
-    allvranges = unique(vcat(collect(values(vranges))...))
-    return vranges, allvranges
-end
-
-function graph_from_data(deps_data, uuid_to_name = Dict{UUID,String}(Types.uuid_julia=>"julia"))
-    uuid(p) = storeuuid(p, uuid_to_name)
-    # deps = DepsGraph(uuid_to_name)
-    fixed = Dict(Types.uuid_julia => Fixed(VERSION))
-    all_versions = Dict{UUID,Set{VersionNumber}}(fp => Set([fx.version]) for (fp,fx) in fixed)
-    all_deps = Dict{UUID,Dict{VersionRange,Dict{String,UUID}}}(fp => Dict(VersionRange(fx.version)=>Dict()) for (fp,fx) in fixed)
-    all_compat = Dict{UUID,Dict{VersionRange,Dict{String,VersionSpec}}}(fp => Dict(VersionRange(fx.version)=>Dict()) for (fp,fx) in fixed)
-
-    deps = Dict{String,Dict{VersionNumber,Dict{String,VersionSpec}}}()
-    for d in deps_data
-        p, vn, r = d[1], d[2], d[3:end]
-        if !haskey(deps, p)
-            deps[p] = Dict{VersionNumber,Dict{String,VersionSpec}}()
-        end
-        if !haskey(deps[p], vn)
-            deps[p][vn] = Dict{String,VersionSpec}()
-        end
-        isempty(r) && continue
-        rp = r[1]
-        rvs = VersionSpec(r[2:end])
-        deps[p][vn][rp] = rvs
-    end
-    for p in keys(deps)
-        u = uuid(p)
-        all_versions[u] = Set(keys(deps[p]))
-        srtvers = sort!(collect(keys(deps[p])))
-
-        deps_pkgs = Dict{String,Set{VersionNumber}}()
-        for (vn,vreq) in deps[p], rp in keys(vreq)
-            push!(get!(deps_pkgs, rp, Set{VersionNumber}()), vn)
-        end
-        vranges, allvranges = gen_versionranges(deps_pkgs, srtvers)
-        all_deps[u] = Dict{VersionRange,Dict{String,UUID}}(VersionRange()=>Dict{String,UUID}("julia"=>Types.uuid_julia))
-        for vrng in allvranges
-            all_deps[u][vrng] = Dict{String,UUID}()
-            for (rp,vvr) in vranges
-                vrng ∈ vvr || continue
-                all_deps[u][vrng][rp] = uuid(rp)
-            end
-        end
-
-        deps_reqs = Dict{Pair{String,VersionSpec},Set{VersionNumber}}()
-        for (vn,vreq) in deps[p], (rp,rvs) in vreq
-            push!(get!(deps_reqs, (rp=>rvs), Set{VersionNumber}()), vn)
-        end
-        vranges, allvranges = gen_versionranges(deps_reqs, srtvers)
-        all_compat[u] = Dict{VersionRange,Dict{String,VersionSpec}}()
-        for vrng in allvranges
-            all_compat[u][vrng] = Dict{String,VersionSpec}()
-            for (req,vvr) in vranges
-                vrng ∈ vvr || continue
-                rp,rvs = req
-                all_compat[u][vrng][rp] = rvs
-            end
-        end
-    end
-    return Graph(all_versions, all_deps, all_compat, uuid_to_name, Requires(), fixed, VERBOSE)
-end
-function reqs_from_data(reqs_data, graph::Graph)
-    reqs = Dict{UUID,VersionSpec}()
-    function uuid_check(p)
-        uuid = pkguuid(p)
-        @assert graph.data.uuid_to_name[uuid] == p
-        return uuid
-    end
-    for r in reqs_data
-        p = uuid_check(r[1])
-        reqs[p] = VersionSpec(r[2:end])
-    end
-    reqs
-end
-function sanity_tst(deps_data, expected_result; pkgs=[])
-    if VERBOSE
-        println()
-        @info("sanity check")
-        # @show deps_data
-        # @show pkgs
-    end
-    graph = graph_from_data(deps_data)
-    id(p) = pkgID(pkguuid(p), graph)
-    result = sanity_check(graph, Set(pkguuid(p) for p in pkgs), VERBOSE)
-
-    length(result) == length(expected_result) || return false
-    expected_result_uuid = [(id(p), vn) for (p,vn) in expected_result]
-    for r in result
-        r ∈ expected_result_uuid || return  false
-    end
-    return true
-end
-sanity_tst(deps_data; kw...) = sanity_tst(deps_data, []; kw...)
-
-function resolve_tst(deps_data, reqs_data, want_data = nothing; clean_graph = false)
-    if VERBOSE
-        println()
-        @info("resolving")
-        # @show deps_data
-        # @show reqs_data
-    end
-    graph = graph_from_data(deps_data)
-    reqs = reqs_from_data(reqs_data, graph)
-    add_reqs!(graph, reqs)
-    simplify_graph!(graph, clean_graph = clean_graph)
-    want = resolve(graph)
-    return want == wantuuids(want_data)
-end
-
 @testset "schemes" begin
     VERBOSE && @info("SCHEME 1")
     ## DEPENDENCY SCHEME 1: TWO PACKAGES, DAG
@@ -328,7 +168,6 @@ end
     deps_data = Any[
         ["A", v"1", "B", "2-*"],
         ["A", v"1", "C", "2-*"],
-        # ["A", v"1", "julia", "10"],
         ["A", v"2", "B", "1"],
         ["A", v"2", "C", "1"],
         ["B", v"1", "C", "2-*"],
@@ -557,22 +396,312 @@ end
     ]
     want_data = Dict("A"=>v"1", "B"=>v"2", "C"=>v"2", "D"=>v"2", "E"=>v"2")
     @test resolve_tst(deps_data, reqs_data, want_data)
+
+
+
+    VERBOSE && @info("SCHEME 11")
+    ## DEPENDENCY SCHEME 11: FOUR PACKAGES, WITH AN INCONSISTENCY
+    ## ref Pkg.jl issue #2740
+    deps_data = Any[
+        ["A", v"1", "C", "1"],
+        ["A", v"2", "C", "2"],
+        ["A", v"2", "D", "1"],
+        ["B", v"1", "D", "1"],
+        ["B", v"2", "D", "2"],
+        ["C", v"1", "D", "1"],
+        ["C", v"1", "B", "1"],
+        ["C", v"2", "D", "2"],
+        ["C", v"2", "B", "2"],
+        ["D", v"1"],
+        ["D", v"2"],
+    ]
+
+    @test sanity_tst(deps_data, [("A", v"2")])
+
+    # require A & B, any version (must use the highest non-inconsistent)
+    reqs_data = Any[
+        ["A", "*"],
+        ["B", "*"],
+    ]
+    want_data = Dict("A"=>v"1", "B"=>v"1", "C"=>v"1", "D"=>v"1")
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+
+    VERBOSE && @info("SCHEME 12")
+    ## DEPENDENCY SCHEME 12: TWO PACKAGES, DAG, WEAK DEPENDENCY
+    deps_data = Any[
+        ["A", v"1", "B", "1-*", :weak],
+        ["A", v"2", "B", "2-*", :weak],
+        ["B", v"1"],
+        ["B", v"2"]
+    ]
+
+    @test sanity_tst(deps_data)
+    @test sanity_tst(deps_data, pkgs=["A", "B"])
+    @test sanity_tst(deps_data, pkgs=["B"])
+    @test sanity_tst(deps_data, pkgs=["A"])
+
+    # require just B
+    reqs_data = Any[
+        ["B", "*"]
+    ]
+    want_data = Dict("B"=>v"2")
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require just A
+    reqs_data = Any[
+        ["A", "*"]
+    ]
+    want_data = Dict("A"=>v"2")
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require A and B
+    reqs_data = Any[
+        ["A", "*"],
+        ["B", "*"]
+    ]
+    want_data = Dict("A"=>v"2", "B"=>v"2")
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require A and B, invompatible versions
+    reqs_data = Any[
+        ["A", "2-*"],
+        ["B", "1"]
+    ]
+    @test_throws ResolverError resolve_tst(deps_data, reqs_data, want_data)
+
+
+    VERBOSE && @info("SCHEME 13")
+    ## DEPENDENCY SCHEME 13: LIKE 9 (SIX PACKAGES, DAG), WITH SOME WEAK DEPENDENCIES
+    deps_data = Any[
+        ["A", v"1"],
+        ["A", v"2"],
+        ["A", v"3"],
+        ["B", v"1", "A", "1"],
+        ["B", v"2", "A", "*"],
+        ["C", v"1", "A", "2", :weak],
+        ["C", v"2", "A", "2-*"],
+        ["D", v"1", "B", "1-*"],
+        ["D", v"2", "B", "2-*", :weak],
+        ["E", v"1", "D", "*"],
+        ["F", v"1", "A", "1-2"],
+        ["F", v"1", "E", "*"],
+        ["F", v"2", "C", "2-*"],
+        ["F", v"2", "E", "*"],
+    ]
+
+    @test sanity_tst(deps_data)
+
+    # require just F
+    reqs_data = Any[
+        ["F", "*"]
+    ]
+    want_data = Dict("A"=>v"3", "C"=>v"2",
+                    "D"=>v"2", "E"=>v"1", "F"=>v"2")
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require just F, lower version
+    reqs_data = Any[
+        ["F", "1"]
+    ]
+    want_data = Dict("A"=>v"2", "D"=>v"2",
+                    "E"=>v"1", "F"=>v"1")
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require F and B; force lower B version -> must bring down F, A, and D versions too
+    reqs_data = Any[
+        ["F", "*"],
+        ["B", "1"]
+    ]
+    want_data = Dict("A"=>v"1", "B"=>v"1", "D"=>v"1",
+                    "E"=>v"1", "F"=>v"1")
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require F and D; force lower D version -> must not bring down F version, and bring in B
+    reqs_data = Any[
+        ["F", "*"],
+        ["D", "1"]
+    ]
+    want_data = Dict("A"=>v"3", "B"=>v"2", "C"=>v"2",
+                    "D"=>v"1", "E"=>v"1", "F"=>v"2")
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require F and C; force lower C version -> must bring down F and A versions
+    reqs_data = Any[
+        ["F", "*"],
+        ["C", "1"]
+    ]
+    want_data = Dict("A"=>v"2", "C"=>v"1",
+                    "D"=>v"2", "E"=>v"1", "F"=>v"1")
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+
+    VERBOSE && @info("SCHEME 14")
+    ## DEPENDENCY SCHEME 14: A NASTY GRAPH WITH A LOCAL OPTIMUM
+    ## (REDUCED VERSION OF REALISTIC SCHEME 17 BELOW, ref Pkg.jl issue #3232)
+    deps_data = Any[
+        ["A", v"1", "X", "*"],
+        ["B", v"1"],
+        ["B", v"2"],
+        ["C", v"1"],
+        ["C", v"2", "G", "2"],
+        ["C", v"2", "H", "*"],
+        ["D", v"1"],
+        ["D", v"2", "C", "*"],
+        ["Y", v"0.1"],
+        ["Y", v"0.2.1", "B", "1"],
+        ["Y", v"0.2.2"],
+        ["X", v"0.1", "Y", "0.1"],
+        ["X", v"0.2", "Y", "0.2"],
+        ["E", v"1", "X", "0.1"],
+        ["F", v"1", "I", "1"],
+        ["G", v"1", "E", "*"],
+        ["G", v"2"],
+        ["H", v"1", "B", "*"],
+        ["H", v"1", "F", "*"],
+        ["H", v"1", "I", "*"],
+        ["I", v"1"],
+        ["I", v"2"],
+    ]
+
+    @test sanity_tst(deps_data)
+
+    # require A and D
+    reqs_data = Any[
+        ["A", "*"],
+        ["D", "*"],
+    ]
+    want_data = Dict(
+        "A"=>v"1",
+        "B"=>v"2",
+        "C"=>v"2",
+        "D"=>v"2",
+        "Y"=>v"0.2.2",
+        "X"=>v"0.2",
+        "F"=>v"1",
+        "G"=>v"2",
+        "H"=>v"1",
+        "I"=>v"1",
+    )
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require just D
+    reqs_data = Any[
+        ["D", "*"],
+    ]
+    want_data = Dict(
+        "B"=>v"2",
+        "C"=>v"2",
+        "D"=>v"2",
+        "F"=>v"1",
+        "G"=>v"2",
+        "H"=>v"1",
+        "I"=>v"1",
+    )
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require just A
+    reqs_data = Any[
+        ["A", "*"],
+    ]
+    want_data = Dict(
+        "A"=>v"1",
+        "Y"=>v"0.2.2",
+        "X"=>v"0.2",
+    )
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require A, D, and lower version of Y
+    reqs_data = Any[
+        ["A", "*"],
+        ["D", "*"],
+        ["Y", "0.2.1"]
+    ]
+    want_data = Dict(
+        "A"=>v"1",
+        "B"=>v"1",
+        "C"=>v"2",
+        "D"=>v"2",
+        "Y"=>v"0.2.1",
+        "X"=>v"0.2",
+        "F"=>v"1",
+        "G"=>v"2",
+        "H"=>v"1",
+        "I"=>v"1",
+    )
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+
+    VERBOSE && @info("SCHEME 15")
+    ## DEPENDENCY SCHEME 15: A GRAPH WITH A WEAK DEPENDENCE
+    ## (REDUCED VERSION OF A REALISTIC SCHEME, ref Pkg.jl issue #4030)
+    deps_data = Any[
+        ["A", v"1"],
+        ["A", v"2", "C", "*"],
+        ["B", v"1", "D", "1", :weak],
+        ["C", v"1", "E", "*"],
+        ["C", v"2", "E", "*"],
+        ["C", v"2", "B", "1"],
+        ["E", v"1", "D", "1"],
+        ["E", v"2", "F", "1"],
+        ["F", v"1", "D", "*"],
+        ["D", v"1"],
+        ["D", v"2"],
+    ]
+
+    @test sanity_tst(deps_data)
+
+    reqs_data = Any[
+        ["A", "*"],
+    ]
+    want_data = Dict(
+        "A" => v"2",
+        "B" => v"1",
+        "C" => v"2",
+        "D" => v"1",
+        "E" => v"2",
+        "F" => v"1",
+    )
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
 end
 
 @testset "realistic" begin
-    VERBOSE && @info("SCHEME REALISTIC")
-    ## DEPENDENCY SCHEME 11: A REALISTIC EXAMPLE
+    VERBOSE && @info("SCHEME REALISTIC 1")
+    ## DEPENDENCY SCHEME 15: A REALISTIC EXAMPLE
     ## ref Julia issue #21485
 
-    include("resolvedata1.jl")
+    tmp = mktempdir()
+    Pkg.PlatformEngines.unpack(joinpath(@__DIR__, "resolvedata.tar.gz"), tmp; verbose=false)
+
+
+    include(joinpath(tmp, "resolvedata1.jl"))
 
     @test sanity_tst(ResolveData.deps_data, ResolveData.problematic_data)
     @test resolve_tst(ResolveData.deps_data, ResolveData.reqs_data, ResolveData.want_data)
+
+    VERBOSE && @info("SCHEME REALISTIC 2")
+    ## DEPENDENCY SCHEME 16: A LARGER, MORE DIFFICULT REALISTIC EXAMPLE
+    ## ref Pkg.jl issue #1949
+
+    include(joinpath(tmp, "resolvedata2.jl"))
+
+    @test sanity_tst(ResolveData2.deps_data, ResolveData2.problematic_data)
+    @test resolve_tst(ResolveData2.deps_data, ResolveData2.reqs_data, ResolveData2.want_data)
+
+    VERBOSE && @info("SCHEME REALISTIC 3")
+    ## DEPENDENCY SCHEME 17: AN EVEN LARGER, MORE DIFFICULT REALISTIC EXAMPLE
+    ## ref Pkg.jl issue #3232
+
+    include(joinpath(tmp, "resolvedata3.jl"))
+
+    @test sanity_tst(ResolveData3.deps_data, ResolveData3.problematic_data)
+    @test resolve_tst(ResolveData3.deps_data, ResolveData3.reqs_data, ResolveData3.want_data)
 end
 
 @testset "nasty" begin
     VERBOSE && @info("SCHEME NASTY")
-    ## DEPENDENCY SCHEME 12: A NASTY CASE
+    ## DEPENDENCY SCHEME 16: A NASTY CASE
 
     include("NastyGenerator.jl")
     deps_data, reqs_data, want_data, problematic_data = NastyGenerator.generate_nasty(5, 20, q=20, d=4, sat = true)
@@ -586,4 +715,66 @@ end
     @test_throws ResolverError resolve_tst(deps_data, reqs_data)
 end
 
+#=
+@testset "Resolving for another version of Julia" begin
+    append!(empty!(Pkg.Types.STDLIBS_BY_VERSION), HistoricalStdlibVersions.STDLIBS_BY_VERSION)
+    temp_pkg_dir() do dir
+        function find_by_name(versions, name)
+            idx = findfirst(p -> p.name == name, versions)
+            if idx === nothing
+                return nothing
+            end
+            return versions[idx]
+        end
+
+        # First, we're going to resolve for specific versions of Julia, ensuring we get the right dep versions:
+        Pkg.Registry.download_default_registries(Pkg.stdout_f())
+        ctx = Pkg.Types.Context(;julia_version=v"1.5")
+        versions, deps = Pkg.Operations._resolve(ctx.io, ctx.env, ctx.registries, [
+            Pkg.Types.PackageSpec(name="MPFR_jll", uuid=Base.UUID("3a97d323-0669-5f0c-9066-3539efd106a3")),
+        ], Pkg.Types.PRESERVE_TIERED, ctx.julia_version)
+        gmp = find_by_name(versions, "GMP_jll")
+        @test gmp !== nothing
+        @test gmp.version.major == 6 && gmp.version.minor == 1
+        ctx = Pkg.Types.Context(;julia_version=v"1.6")
+        versions, deps = Pkg.Operations._resolve(ctx.io, ctx.env, ctx.registries, [
+            Pkg.Types.PackageSpec(name="MPFR_jll", uuid=Base.UUID("3a97d323-0669-5f0c-9066-3539efd106a3")),
+        ], Pkg.Types.PRESERVE_TIERED, ctx.julia_version)
+        gmp = find_by_name(versions, "GMP_jll")
+        @test gmp !== nothing
+        @test gmp.version.major == 6 && gmp.version.minor == 2
+
+        # We'll also test resolving an "impossible" manifest; one that requires two package versions that
+        # are not both loadable by the same Julia:
+        ctx = Pkg.Types.Context(;julia_version=nothing)
+        versions, deps = Pkg.Operations._resolve(ctx.io, ctx.env, ctx.registries, [
+            # This version of GMP only works on Julia v1.6
+            Pkg.Types.PackageSpec(name="GMP_jll", uuid=Base.UUID("781609d7-10c4-51f6-84f2-b8444358ff6d"), version=v"6.2.0"),
+            # This version of MPFR only works on Julia v1.5
+            Pkg.Types.PackageSpec(name="MPFR_jll", uuid=Base.UUID("3a97d323-0669-5f0c-9066-3539efd106a3"), version=v"4.0.2"),
+        ], Pkg.Types.PRESERVE_TIERED, ctx.julia_version)
+        gmp = find_by_name(versions, "GMP_jll")
+        @test gmp !== nothing
+        @test gmp.version.major == 6 && gmp.version.minor == 2
+        mpfr = find_by_name(versions, "MPFR_jll")
+        @test mpfr !== nothing
+        @test mpfr.version.major == 4 && mpfr.version.minor == 0
+    end
+    empty!(Pkg.Types.STDLIBS_BY_VERSION)
+end
+=#
+
+@testset "Stdlib resolve smoketest" begin
+    # All stdlibs should be installable and resolvable
+    temp_pkg_dir() do dir
+        Pkg.activate(temp=true)
+        Pkg.add(map(first, values(Pkg.Types.load_stdlib())))    # add all stdlibs
+        iob = IOBuffer()
+        Pkg.resolve(io = iob)
+        str = String(take!(iob))
+        @test occursin(r"No Changes to .*Project.toml", str)
+        @test occursin(r"No Changes to .*Manifest.toml", str)
+    end
+end
+
 end # module
