@@ -7,10 +7,9 @@ makedocs(
     modules = [SparseArrays],
     sitename = "SparseArrays",
     pages = Any[
-        "SparseArrays" => "index.md"
+        "SparseArrays" => "index.md",
         ];
-    # strict = true,
-    strict = Symbol[:doctest],
+    warnonly = [:missing_docs, :cross_references],
     )
 
-deploydocs(repo = "github.com/JuliaLang/SparseArrays.jl.git")
+deploydocs(repo = "github.com/JuliaSparse/SparseArrays.jl.git")
